# _*_ coding: utf-8 _*_
#
# hspylib v0.11.172
#
# Package: main.hspylib.modules.application
"""Package initialization."""

__all__ = [
  'application', 
  'argparse', 
  'exit_hooks', 
  'exit_status', 
  'parser_action', 
  'version'
]
__version__ = '0.11.172'
